-----------------------------------------------------------------------------------------------------------------------
-- FILE			: reader.vhd
-- TITLE		:
-- PROJECT		:
-- AUTHOR		: Lucio
-- DESCRIPTION	: Implements the reader interface for VGA Module
-- NOTES		:
-- REVISION		: $Id$
-- DEPENDENCIES	: To be used within vga_out_16bit module
-----------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------------------
-- Libraries
-----------------------------------------------------------------------------------------------------------------------
library ieee;
	use ieee.std_logic_1164.all;
	use ieee.std_logic_unsigned.all;
	use ieee.std_logic_arith.all;

-----------------------------------------------------------------------------------------------------------------------
-- Ports                                                                                                                					-- Entidade --
-----------------------------------------------------------------------------------------------------------------------
entity reader is
generic (
	C_SLV_DWIDTH                   : integer              := 32;
  C_MST_AWIDTH                   : integer              := 32;
  C_MST_DWIDTH                   : integer              := 64
);
port
(
	-- Clocks and resets
	PLB_Clk					: in	std_logic;
	PLB_Rst					: in	std_logic;
	pxl_clk					: in	std_logic;
	burst_length		: in  std_logic_vector(0 to 11);
	-- PLB Master Interface
	IP2Bus_MstRd_Req               : out std_logic;
	IP2Bus_MstWr_Req               : out std_logic;
	IP2Bus_Mst_Addr                : out std_logic_vector(0 to C_MST_AWIDTH-1);
	IP2Bus_Mst_BE                  : out std_logic_vector(0 to C_MST_DWIDTH/8-1);
	IP2Bus_Mst_Length              : out std_logic_vector(0 to 11);
	IP2Bus_Mst_Type                : out std_logic;
	IP2Bus_Mst_Lock                : out std_logic;
	IP2Bus_Mst_Reset               : out std_logic;
	Bus2IP_Mst_CmdAck              : in  std_logic;
	Bus2IP_Mst_Cmplt               : in  std_logic;
	Bus2IP_Mst_Error               : in  std_logic;
	Bus2IP_Mst_Rearbitrate         : in  std_logic;
	Bus2IP_Mst_Cmd_Timeout         : in  std_logic;
	Bus2IP_MstRd_d                 : in  std_logic_vector(0 to C_MST_DWIDTH-1);
	Bus2IP_MstRd_rem               : in  std_logic_vector(0 to C_MST_DWIDTH/8-1);
	Bus2IP_MstRd_sof_n             : in  std_logic;
	Bus2IP_MstRd_eof_n             : in  std_logic;
	Bus2IP_MstRd_src_rdy_n         : in  std_logic;
	Bus2IP_MstRd_src_dsc_n         : in  std_logic;
	IP2Bus_MstRd_dst_rdy_n         : out std_logic;
	IP2Bus_MstRd_dst_dsc_n         : out std_logic;
	-- Line Buffer interface
	ln_buf_we						: out std_logic;						-- Write Enable
	ln_buf_col					: out std_logic_vector(9 downto 0);		-- Write Column
	ln_buf_pxl					: out std_logic_vector(63 downto 0);	-- Write Value
	-- VGA Timer
	reset_timer					: out std_logic;						-- Reset timer
	new_line						: in  std_logic;						-- Request new line
	-- Control
	start								: in	std_logic;
	color_depth					: in	std_logic_vector(1 downto 0);	-- Color depth
	base_addr						: in	std_logic_vector(31 downto 0)	-- Buffer base address
);
end reader;

-----------------------------------------------------------------------------------------------------------------------
-- Architecture
-----------------------------------------------------------------------------------------------------------------------
architecture behavior of  reader is

	-- -------------------------------
	-- Constants
	-- -------------------------------
	type FSM_TYPE is (
		PREIDLE,
		IDLE,
		RD_BURST,
		RD_BURST_CTRL,
		RD_BURST_END
	);

	-- -------------------------------
	-- Signal Declarations
	-- -------------------------------
	signal fsm_state		: FSM_TYPE;								-- Finite State Machine

	signal tmp_reg			: std_logic_vector(63 downto 0);		-- Value read from memory

	signal pxl_cnt			: std_logic_vector(9 downto 0);			-- Counts pixels
	signal line_cnt			: std_logic_vector(8 downto 0);			-- Count lines to reset address
	signal addr					: std_logic_vector(31 downto 0);		-- Read Address

	signal start_rst		: std_logic := '0';						-- Indicates that VGA Timer must be reset after next line is buffered
	signal go_rst				: std_logic;							-- One line has been buffered, ok to rst VGA Timer
	signal rst_cnt			: std_logic_vector(15 downto 0);		-- Used to drive reset_timer long enough
	
	signal burst_size		: std_logic_vector(0 to 11) := (others => '0');

	-- -------------------------------
	-- Components
	-- -------------------------------

begin

	-- -------------------------------
	-- Port Mapping
	-- -------------------------------

	-- -------------------------------
	-- Processes
	-- -------------------------------
	process(PLB_Clk)
	begin

		if PLB_Clk'event and PLB_Clk = '1' then

			if PLB_Rst = '1' then
				IP2Bus_MstRd_Req              <= '0';
				IP2Bus_MstWr_Req              <= '0';
				IP2Bus_Mst_Addr               <= (others => '0');
				IP2Bus_Mst_BE                 <= (others => '0');
				IP2Bus_Mst_Length             <= (others => '0');
				IP2Bus_Mst_Type               <= '0';
				IP2Bus_Mst_Lock               <= '0';
				IP2Bus_Mst_Reset              <= '0';
				IP2Bus_MstRd_dst_rdy_n        <= '1';
				IP2Bus_MstRd_dst_dsc_n        <= '1';
				fsm_state											<= PREIDLE;
				pxl_cnt												<= (others => '0');
				line_cnt											<= (others => '0');
				addr													<= base_addr;
				ln_buf_we											<= '0';
				start_rst											<= '1';
				burst_size										<= conv_std_logic_vector(160, 12); --160

			else
				case fsm_state is
				when PREIDLE =>
					go_rst		<=	'0';
					if (new_line = '1' or start_rst = '1') and start = '1' then
						fsm_state	<=	IDLE;
					end if;

				when IDLE =>
					if color_depth = "11" then
						burst_size	<=	conv_std_logic_vector(320, 12); --320
					elsif color_depth = "10" then
						burst_size	<=	conv_std_logic_vector(160, 12); --160
					elsif color_depth = "00" then
						burst_size	<=	conv_std_logic_vector(80, 12);  --80
					end if;
					IP2Bus_MstRd_Req              <= '0';
					IP2Bus_MstWr_Req              <= '0';
					IP2Bus_Mst_Addr               <= (others => '0');
					IP2Bus_Mst_BE                 <= (others => '0');
					IP2Bus_Mst_Length             <= (others => '0');
					IP2Bus_Mst_Type               <= '0';
					ln_buf_we					<=	'0';
					fsm_state					<=	RD_BURST;

				when RD_BURST =>
					IP2Bus_MstRd_Req					<= '1';
					IP2Bus_Mst_Type						<= '1';
					IP2Bus_Mst_Addr						<= addr;
--					if burst_length > 0 then
--						IP2Bus_Mst_Length 				<= burst_length;
--					else
--						IP2Bus_Mst_Length 				<= "000010000000";
--					end if;
					IP2Bus_Mst_Length 				<= conv_std_logic_vector(conv_integer(burst_size) * 8, 12);
					IP2Bus_MstRd_dst_rdy_n		<= '0';
					if (Bus2IP_Mst_CmdAck = '1') then
						fsm_state	<= RD_BURST_CTRL;
						
					end if;

				when RD_BURST_CTRL =>
					IP2Bus_Mst_Type					<=  '0';
					IP2Bus_MstRd_Req				<=	'0';
					IP2Bus_Mst_Addr					<= (others => '0');
					ln_buf_we								<=	not(Bus2IP_MstRd_src_rdy_n);
					if (Bus2IP_MstRd_src_rdy_n = '0') then
						ln_buf_col	<=	pxl_cnt;
						pxl_cnt			<=	pxl_cnt + '1';
						ln_buf_pxl	<=	Bus2IP_MstRd_d;
						addr				<=	addr + 8;
						if pxl_cnt = burst_size - 1 then
							pxl_cnt	<=	(others => '0');
							if start_rst = '1' then
								start_rst	<=	'0';
								go_rst		<=	'1';
							else
								go_rst	<=	'0';
							end if;
							if line_cnt = "111011111" then			--479
								line_cnt	<=	(others => '0');
								addr			<=	base_addr;
							else
								line_cnt	<=	line_cnt + '1';
							end if;
							fsm_state	<=	RD_BURST_END;
						else
							fsm_state	<=	RD_BURST_CTRL;
						end if;
						if (Bus2IP_MstRd_eof_n = '0') then
							IP2Bus_MstRd_dst_rdy_n	<= 	'1';
							fsm_state	<=	RD_BURST_END;
						else
							fsm_state	<=	RD_BURST_CTRL;
						end if;
					end if;
					

				when RD_BURST_END =>
					--IP2Bus_MstRd_dst_rdy_n		<= '1';
					ln_buf_we		<=	'0';
					fsm_state		<=	PREIDLE;

				when others =>
					fsm_state	<=	PREIDLE;

				end case;
			end if;
		end if;
	end process;

	-- Controls VGA Timer Rst
	process(PLB_Clk)
	begin
		if PLB_Clk'event and PLB_Clk = '1' then
			if PLB_Rst = '1' then
				rst_cnt	<= (others => '0');
			else
				if go_rst = '1' then
					rst_cnt	<= "1111111111111111";
				else
					rst_cnt	<= '0' & rst_cnt(15 downto 1);
				end if;
			end if;
		end if;
	end process;

	-- -------------------------------
	-- Asynchronous Attributions
	-- -------------------------------

	-- PLB Bus
	
	-- VGA Timer
	reset_timer	<=	rst_cnt(0);

end behavior;