-----------------------------------------------------------------------------------------------------------------------
-- FILE			: vga_timer.vhd
-- TITLE		:
-- PROJECT		:
-- AUTHOR		: Lucio
-- DESCRIPTION	: Implements a 640x480 VGA Timer
-- NOTES		:
-- REVISION		: $Id$
-- DEPENDENCIES	: To be used within vga_out_16bit module
-----------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------------------
-- Libraries
-----------------------------------------------------------------------------------------------------------------------
library ieee;
	use ieee.std_logic_1164.all;
	use ieee.std_logic_unsigned.all;
	use ieee.std_logic_arith.all;

-----------------------------------------------------------------------------------------------------------------------
-- Ports                                                                                                                					-- Entidade --
-----------------------------------------------------------------------------------------------------------------------
entity vga_timer is
generic
(
	BUF_ADDR_SIZE	: integer	:=	10;			-- Buffer's address size in bits
	V_PW			: integer	:=	2;			-- Vertical pulse width
	V_BP			: integer	:=	31; --29	-- Vertical back porch
	V_DT			: integer	:=	480;		-- Vertical display time
	V_FP			: integer	:=	11;	--9		-- Vertical front porch
	V_PT			: integer	:=	524;		-- Vertical pulse time
	H_PW			: integer	:=	96;			-- Horizontal pulse width
	H_BP			: integer	:=	48;			-- Horizontal back porch
	H_DT			: integer	:=	640;		-- Horizontal display time
	H_FP			: integer	:=	16;			-- Horizontal front porch
	H_PT			: integer	:=	800			-- Horizontal pulse time
);
port
(
	-- Clocks and resets -----------------------------------------------------------------------------------------------
	reset			: in	std_logic;										-- rst signal
	pixel_clk		: in	std_logic;										-- pixel clock - 25MHz
	-- To read_module --------------------------------------------------------------------------------------------------
	new_line		: out	std_logic;										-- requests a new line to be buffered
	-- To VGA output ---------------------------------------------------------------------------------------------------
	h_sync			: out	std_logic;										-- horizontal sync
	v_sync			: out	std_logic;										-- vertical sync
	-- To line_buffer --------------------------------------------------------------------------------------------------
	buf_addr_rd		: out	std_logic_vector(BUF_ADDR_SIZE - 1 downto 0);	-- address to read from buffer
	rd_buf			: out	std_logic;
	is_visible		: out	std_logic										-- indicates if pixel is visible or not
);
end vga_timer;

-----------------------------------------------------------------------------------------------------------------------
-- Architecture
-----------------------------------------------------------------------------------------------------------------------
architecture behavior of  vga_timer is


	-- -------------------------------
	-- Signal Declarations
	-- -------------------------------
	signal h_counter		: integer range 0 to H_PT - 1	:= 0;			-- horizontal pixels counter
	signal v_counter		: integer range 0 to V_PT - 1	:= 0;			-- vertical pixels counter

	signal v_sync_int		: std_logic := '0';								-- v_sync for internal use
	signal h_sync_int		: std_logic := '0';								-- h_sync for internal use
	signal v_display		: std_logic := '1';								-- set to '1' when inside vertical display time
	signal buf_addr_int		: std_logic_vector(BUF_ADDR_SIZE - 1 downto 0);	-- buffer address for internal use

	signal v_visible		: std_logic	:= '0';								-- pixel visibility
	signal h_visible		: std_logic	:= '0';								-- pixel visibility


	-- -------------------------------
	-- Components
	-- -------------------------------

begin

	-- -------------------------------
	-- Port Mapping
	-- -------------------------------

	-- -------------------------------
	-- Processes
	-- -------------------------------
	-- Generates v_sync
	process(h_sync_int, pixel_clk)
	begin
		if reset = '1' then
			v_counter	<=	0;
			v_sync_int	<=	'0';
			v_visible	<=	'0';
		elsif h_sync_int'event and h_sync_int = '1' then
			-- Controls v_counter
			if v_counter < V_PT - 1 then
				v_counter	<=	v_counter + 1;
				-- Controls v_sync
				if v_counter < V_PW - 1 then
					v_sync_int	<=	'0';
				elsif v_counter	< V_PW + V_BP - 1 then
					v_sync_int	<=	'1';
				elsif v_counter	< V_PW + V_BP + V_DT - 1 then
					v_visible	<=	'1';
				else
					v_visible	<=	'0';
				end if;
			else
				v_counter	<=	0;
				v_sync_int	<=	'0';
			end if;
		end if;
	end process;

	-- Generates h_sync, is_visible and buf_addr_wr
	process(pixel_clk)
	begin
		if pixel_clk'event and pixel_clk = '1' then
			if reset = '1' then
				h_visible			<=	'0';
				h_counter		<=	0;
				h_sync_int		<=	'0';
				new_line		<=	'0';
				rd_buf			<=	'0';
				buf_addr_int	<=	(others => '0');
			else
				-- Controls h_counter
				if h_counter < H_PT - 1 then
					h_counter	<=	h_counter + 1;
					-- Controls h_sync and buf_addr_rd
					if h_counter < H_PW - 1 then
						h_sync_int		<=	'0';
						new_line		<=	'0';
					elsif h_counter	< H_PW + H_BP - 1 then
						h_sync_int		<=	'1';
						buf_addr_int	<=	(others => '0');
						rd_buf			<=	'1';
					elsif h_counter	< H_PW + H_BP + H_DT - 1 then
						if conv_integer(buf_addr_int) < H_DT - 1 then
							buf_addr_int	<=	buf_addr_int + '1';
						else
							rd_buf	<=	'0';
						end if;
						h_visible		<=	'1';
					else
						h_visible		<=	'0';
						new_line		<=	'1' and v_visible;
					end if;
				else
					h_sync_int	<=	'0';
					h_counter	<=	0;
				end if;


			end if;
		end if;
	end process;

	-- -------------------------------
	-- Asynchronous Attributions
	-- -------------------------------
	is_visible	<=	v_visible and h_visible and not reset;
	v_sync		<=	v_sync_int;
	h_sync		<=	h_sync_int;
	buf_addr_rd	<=	buf_addr_int;

end behavior;